// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - LOBBY [OVERVIEW]
// Match Overview
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Where to BEGIN drawing the info
#define MAPINFO_X				323.0
#define MAPINFO_Y				38.0

#define MAPDESC_X				14.0
#define MAPDESC_Y				285.0
#define MAPDESC_W				505.0
#define MAPDESC_H				252.0

namespace LBOverview
{
	using LBCore;
	using ZTMaterials;
	
	private struct Rect DescRect;				// For description
	
	private struct Rect BackgroundRect;		// Where to draw BG
	bool DirtyBackground;					// BG needs redrawing
	
	private struct Rect ThumbnailRect;		// Where to draw thumbnail
	bool DirtyThumbnail, DirtyName, DirtyDesc, DirtyAuthor;
	
	// Last known values for certain elements
	// If any of these are invalid, our menu is dirty!
	private str LobImage, LobName, LobDesc, LobAuthor;
	private str LobDescWrapped;								// Auto-wrapped
	
	// ------------------------/
	// Initialize!
	// ------------------------/
	
	void LBO_Init()
	{
		// Generate the elements!
		LBO_Generate();
	}
	
	// ------------------------/
	// Cleanup
	// ------------------------/
	
	void LBO_Cleanup()
	{
		ClearHUDMessage(HID_OVERVIEW_BG);
		ClearHUDMessage(HID_OVERVIEW_IMAGE);
		ClearHUDMessage(HID_OVERVIEW_INFO);
		ClearHUDMessage(HID_OVERVIEW_DESC);
	}
	
	// ------------------------/
	// Draw
	// ------------------------/
	
	void LBO_Draw()
	{
		if (LobbyMode != LM_OVERVIEW)
			return;
			
		// Background!
		if (DirtyBackground)
		{
			DirtyBackground = false;
			HUDMessageMaterial(LOBBOVBG, BackgroundRect, HID_OVERVIEW_BG);
		}
		
		// Map pic
		if (DirtyThumbnail)
		{
			DirtyThumbnail = false;
			
			SetFont(LobImage);
			HUDMessageUnscaled(ThumbnailRect.X, ThumbnailRect.y, "a", ALIGN_LEFT, ALIGN_TOP, HID_OVERVIEW_IMAGE);
		}
		
		// Map info
		if (DirtyName || DirtyAuthor)
		{
			DirtyName = false;
			DirtyAuthor = false;
			
			str levName = StrCapitalize( StrParam(n:PRINTNAME_LEVEL) );
			str newText = StrParam(s:"\cU", s:levName, s:"\n", s:LobName, s:"\nBy ", s:LobAuthor, s:"\n\n");
			
			// Difficulty
			newText = StrParam(s:newText, s:"Difficulty: ", d:GameSkill(), s:"\n");
			
			SetFont(FNT_PLAIN);
			HUDMessageUnscaled(BackgroundRect.X + MAPINFO_X, BackgroundRect.Y + MAPINFO_Y, newText, ALIGN_LEFT, ALIGN_TOP, HID_OVERVIEW_INFO);
		}
		
		// Description
		if (DirtyDesc)
		{
			DirtyDesc = false;
			
			SetFont(FNT_PLAIN);
			HUDMessageUnscaled(DescRect.X, DescRect.Y, LobDescWrapped, ALIGN_LEFT, ALIGN_TOP, HID_OVERVIEW_DESC);
		}
	}
	
	// ------------------------/
	// Tick
	// ------------------------/
	
	void LBO_Tick()
	{
		str nm, pc, ds, au;
		
		if (LobbyMode != LM_OVERVIEW)
			return;
			
		// Sync our content up if necessary
		// Tick is done BEFORE draw
		
		nm = GetMapTitle();
		pc = GetMapPicture();
		ds = GetMapDescription();
		au = GetMapAuthor();
		
		// -- Picture
		if (StrCmp(LobImage, pc) != 0)
		{
			LobImage = pc;
			DirtyThumbnail = true;
		}
		
		// -- Name
		if (StrCmp(LobName, nm) != 0)
		{
			LobName = nm;
			DirtyName = true;
		}
		
		// -- Description
		if (StrCmp(LobDesc, ds) != 0)
		{
			LobDesc = ds;
			DirtyDesc = true;
			
			LobDescWrapped = StrWrap(ds, DescRect.W, GLYPHWIDTH_PLAIN);
		}
		
		// -- Author
		if (StrCmp(LobAuthor, au) != 0)
		{
			LobAuthor = au;
			DirtyAuthor = true;
		}
	}
	
	// ------------------------/
	// Generate
	// ------------------------/
	
	private void LBO_Generate(bool ifShown = false)
	{
		if (ifShown && LobbyMode != LM_OVERVIEW)
			return;
			
		DirtyBackground = true;
		DirtyThumbnail = true;
		DirtyName = true;
		DirtyDesc = true;
		DirtyAuthor = true;
		
		// Decide where to draw our background!
		int cenX = LBCore::GetLobbyCenterX();
		int cenY = LBCore::GetLobbyCenter();
		
		cenX -= FixedMul(LOBBOVBG.W, 0.5);
		cenY -= FixedMul(LOBBOVBG.H, 0.5);
		
		SetRect(BackgroundRect, cenX, cenY, PSELBG.W, PSELBG.H);
		
		// Where to draw thumb?
		SetRect(ThumbnailRect, cenX+8.0, cenY+31.0, MPICNONE.W, MPICNONE.H);
		
		// Description box
		SetRect(DescRect, cenX + MAPDESC_X, cenY + MAPDESC_Y, MAPDESC_W, MAPDESC_H);
	}
}