// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - LOBBY [SQUAD]
// Shows info about all players on the server
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#define SQUAD_PAD_TOP				140.0
#define SQUAD_PAD_RIGHT				48.0
#define SQUAD_ITEM_PADDING			8.0

#define SQUAD_BARSTART_Y			16.0

#define SQUAD_HEADER_Y				30.0
#define SQUAD_ITEM_Y				80.0

#define SQUAD_READYPAD_BOTTOM		32.0

#define SQUAD_PERKLEVEL_W			48.0

#define SQUAD_CHECK_INTERVAL		35		// How often to check player info, in tics

struct SquadMemberContainer
{
	int Index;						// This points to the REAL player number
};

namespace LBSquad
{
	using ZTMaterials;
	using LBCore;
	using LBTop;
	using Scrollbars;
	
	// Stored information for the squad members in the list
	private struct SquadMemberContainer SquadMembers[MAXPLAYERS];
	private int SquadMemberSize;
	
	private int ResX, ResY;					// Last known res
	
	private bool DirtyBox;					// Box needs redrawing
	private bool DirtyItems;				// Items need redrawing
	private bool DirtyReady;				// Ready button needs redoing
	
	private bool HoverReady;				// Hovering over ready button?
	private bool WasReady;					// Whether or not WE were ready
	
	private int SquadX, SquadY;				// Positions of menu
	private int SquadItemX, SquadItemY;		// Position of actual items
	private int SquadHeight;				// Height of box
	private int SquadItemCount;				// How many items we can show on-screen
	
	private struct Rect ReadyRect, HeaderRect;
	
	// For the scrollbar
	private struct Rect SquadBarR;
	private struct ScrollBarT SquadBar;
	private int LastBarPos;
	
	// For manually checking players!
	private int CheckTick;
	
	// ------------------------/
	// Initialize!
	// ------------------------/
	
	private void LBS_Init()
	{
		DirtyBox = true;
		DirtyItems = true;
		DirtyReady = true;
		
		LBS_Generate();
		LBS_GenerateMembers();
	}
	
	// ------------------------/
	// Cleanup
	// ------------------------/
	
	private void LBS_Cleanup()
	{
		int HID = HID_LOBBY_SQUADBUT;
		
		// Cleanup the actual buttons
		for (int i=0; i<SquadItemCount; i++)
		{
			// Background
			ClearHUDMessage(HID);
			HID -= 1;
			
			// Text
			ClearHUDMessage(HID);
			HID -= 1;
			
			// Level
			ClearHUDMessage(HID);
			HID -= 1;
		}
		
		// Cleanup ready and header
		ClearHUDMessage(HID_LOBBY_SQUADREADY);
		ClearHUDMessage(HID_LOBBY_SQUADHDR);
		
		// Cleanup the BG
		HUDMessageRectangle(SquadX, SquadY, SquadHeight, HID_LOBBY_SQUADBG, TRADBG1, TRADBG2, TRADBG2, true);
		
		// Cleanup bar
		Bar_Cleanup(SquadBar);
	}
	
	// ------------------------/
	// Draw
	// ------------------------/
	
	private void LBS_Draw()
	{
		bool barWasDirty = ScrollbarDirty(SquadBar);
		
		// Ready button
		if (DirtyReady)
		{
			DirtyReady = false;
			
			int CPN = ConsolePlayerNumber();
			
			if (PlayerState[CPN].Ready || PlayerState[CPN].PerkIndex <= 0)
				HUDMessageMaterial(HoverReady ? LOBBRED4 : LOBBRED3, ReadyRect, HID_LOBBY_SQUADREADY);
			else
				HUDMessageMaterial(HoverReady ? LOBBRED2 : LOBBRED1, ReadyRect, HID_LOBBY_SQUADREADY);
		}
		
		// Box is dirty?
		if (DirtyBox)
		{
			DirtyBox = false;
			
			// Draw the squad box
			HUDMessageRectangle(SquadX, SquadY, SquadHeight, HID_LOBBY_SQUADBG, TRADBG1, TRADBG2, TRADBG2);
			
			// Draw the header
			HUDMessageMaterial(LOBBSQD1, HeaderRect, HID_LOBBY_SQUADHDR);
		}
		
		// -- Draw all of the players --==--==--==--==--==--==--==--==--==--==--==--==
		
		if (DirtyItems)
		{
			DirtyItems = false;
			
			// Is the bar even needed?
			bool wasNeeded = !SquadBar.Disabled;
			bool needBar = (SquadMemberSize > SquadItemCount);
			
			if (wasNeeded != needBar)
			{
				if (needBar)
					Bar_Enable(SquadBar);
				else
					Bar_Disable(SquadBar);
					
				Bar_Reset(SquadBar);
			}
			
			int TextX = (SquadItemX + TRADPRK1.W) - 8.0;
			
			// Possible items in scrollbar range
			int lastItem = SquadMemberSize - SquadItemCount;
			int firstItem = ShaveInt( FixedMul(SquadBar.Pct, lastItem * 1.0) );
			
			int curY = SquadItemY;
			int HID = HID_LOBBY_SQUADBUT;
			
			// Loop through all items that COULD be shown on the menu
			for (int i = firstItem; i < firstItem + SquadItemCount; i++)
			{
				// Outside of list range, clear it
				if (i < 0 || i >= SquadMemberSize)
				{
					ClearHUDMessage(HID);
					ClearHUDMessage(HID-1);
					
					HID -= 2;
					curY += TRADPRK1.H + SQUAD_ITEM_PADDING;
					continue;
				}
				
				// Draw this member!
				// Draw the bar
				HUDMessageMaterialOnly(TRADPRK1, SquadItemX, curY, HID);
				HID --;
				
				// Draw their name
				int TextY = curY + FixedMul(TRADPRK1.H, 0.5);
				
				int pNum = SquadMembers[i].Index;
				
				str ReadyText = PlayerState[pNum].Ready ? "\cDREADY" : "\cINOT READY";
				str butText = StrParam(s:ReadyText, s:" \c-- ", n:pNum + 1);
				
				// Add perk
				str pName = PerkList[ PlayerState[pNum].PerkIndex ].DisplayName;
				butText = StrParam(s:butText, s:" [", s:pName, s:"]");
				
				SetFont(FNT_PLAIN);
				HUDMessageUnscaled(TextX - SQUAD_PERKLEVEL_W, TextY, butText, ALIGN_RIGHT, ALIGN_CENTER, HID);
				HID --;
				
				//-- DRAW THE PERK LEVEL --
				str pLevel = StrParam(d:PlayerState[pNum].PerkLevel);

				SetFont("BIGFONT");
				HUDMessageUnscaled(TextX, TextY, pLevel, ALIGN_RIGHT, ALIGN_CENTER, HID);
				HID --;

				curY += TRADPRK1.H + SQUAD_ITEM_PADDING;
			}
		}
		
		// Draw scrollbar
		if (barWasDirty)
			Bar_Draw(SquadBar);
	}
	
	// ------------------------/
	// Tick
	// ------------------------/
	
	private void LBS_Tick()
	{
		// Res is dirty!
		int RX = ResolutionX();
		int RY = ResolutionY();
		
		if (ResX != RX || ResY != RY)
		{
			ResX = RX;
			ResY = RY;
			LBS_CleanUp();
			LBS_Generate();
		}
		
		// Hovering over ready
		bool MR = Mousing(ReadyRect) && !ScrollBars::barHeld;
		
		if (HoverReady != MR)
		{
			if (MR)
				AmbientSound("trader/perkover", 127);
				
			HoverReady = MR;
			DirtyReady = true;
		}
		
		// OUR ready state changed
		int CPN = ConsolePlayerNumber();
		
		if (PlayerState[CPN].Ready != WasReady)
		{
			WasReady = PlayerState[CPN].Ready;
			DirtyReady = true;
		}
		
		// We received player info, our squads are dirty
		if (GameState.HUD.SquadSyncPending)
		{
			GameState.HUD.SquadSyncPending = false;
			LBS_GenerateMembers();
		}
		
		// Bar moved
		if (LastBarPos != SquadBar.BarY)
		{
			LastBarPos = SquadBar.BarY;
			DirtyItems = true;
		}
		
		CheckTick ++;
		
		if (CheckTick >= SQUAD_CHECK_INTERVAL)
		{
			CheckTick = 0;
			LBS_GenerateMembers();
		}
	}
	
	// ------------------------/
	// Generate the elements for the menu!
	// ------------------------/
	
	private void LBS_Generate()
	{
		ResX = ResolutionX();
		ResY = ResolutionY();
		
		// Decide X position of the SQUAD menu
		SquadX = ((ResX * 1.0) - SQUAD_PAD_RIGHT) - TRADBG1.W;
		
		// Where should we put the ready button?
		int cenX = SquadX + FixedMul(TRADBG1.W, 0.5);
		int rdyX = cenX - FixedMul(LOBBRED1.W, 0.5);
		int rdyY = ((ResY*1.0) - SQUAD_READYPAD_BOTTOM) - LOBBRED1.H;
		
		SetRect(ReadyRect, rdyX, rdyY, LOBBRED1.W, LOBBRED1.H);
		
		// Decide Y position
		SquadY = SQUAD_PAD_TOP;
		
		SquadItemX = ((SquadX + TRADBG1.W) - 8.0) - TRADPRK1.W;
		SquadItemY = SquadY + SQUAD_ITEM_Y;
		
		// Generate header rectangle
		int hedX = (SquadX + TRADBG1.W) - LOBBSQD1.W;
		int hedY = SquadY + SQUAD_HEADER_Y;
		SetRect(HeaderRect, hedX, hedY, LOBBSQD1.W, LOBBSQD1.H);
		
		// Decide HEIGHT of squad box
		SquadHeight = (ResY*1.0) - SquadY;
		int SquadItemHeight = ReadyRect.Y - SquadItemY;
		
		// How many items can we show?
		int itemHeight = TRADPRK1.H + SQUAD_ITEM_PADDING;
		
		SquadItemCount = ShaveInt( FixedDiv(SquadItemHeight, itemHeight) ) - 1;
		
		// -- SETUP SQUAD SCROLLBAR --
		int barX = SquadX + TRADBG1.W;
		SetRect(SquadBarR, barX-2.0, SquadY+SQUAD_BARSTART_Y, TRADSBR1.W, SquadHeight);
		SetScrollbar(SquadBar, SquadBarR, HID_LOBBY_SQUADBAR);
	}
	
	// ------------------------/
	// GENERATE squad members
	// This is a bit costly and loops through each player
	// ------------------------/
	
	private void LBS_GenerateMembers()
	{
		int lastSize = SquadMemberSize;
		
		SquadMemberSize = 0;
		
		for (int i=0; i<MAXPLAYERS; i++)
		{
			// Does this work on the client end? May need changing later
			if (PlayerValid(i))
			{
				SquadMembers[SquadMemberSize].Index = i;
				SquadMemberSize ++;
			}
		}
		
		DirtyItems = true;
		DirtyReady = true;

		// Someone joined or left, reset scrollbar
		if (lastSize != SquadMemberSize)
			Bar_Reset(SquadBar);
	}
	
	// ------------------------/
	// Left click
	// ------------------------/
	
	private void LBS_MouseClick()
	{
		// Scrollbar functionality if we're hovering over it
		if (ScrollBars::Bar_Hovered(SquadBar))
		{
			Scrollbars::Bar_AttemptSet(SquadBar);
			return;
		}
		
		// This may not reach the server
		// Not quite important enough to make it reliable though
		
		int CPN = ConsolePlayerNumber();
		
		if (HoverReady)
		{
			AmbientSound("trader/perkclick", 127);
			
			if (PlayerState[CPN].PerkIndex > 0)
				NamedRequestScriptPuke("ReadyUp",0,0,0,0);
			else
				LT_DoPerkError();
		}
	}
}