// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ZOMBIE TORRENT 2 - ZOMBIE LIST
// Contains core code for the actual zombie list
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// Zombie's body shape
// This is used to spawn proper zombies in appropriate spots
enum ZombieBuild
{
	BUILD_HUMAN,					// Plain human build (imp, zombieman, etc.)
	BUILD_MIDGET,					// Dogs, anything that crawls on the ground
	BUILD_LANKY,					// Revenant, facemaw, anything that's tall
	BUILD_BULKY,					// Chaingunners, anything really bulky and wide
	BUILD_HEAD,						// Flying heads, or anything skull-sized
	
	// Used by spawners
	BUILD_ALL,						// Any zombie can spawn here
};

// Zombie entry in the list
struct ZombieEntry
{
	str Class;						// The actual DECORATE class name
	str DisplayName;				// Friendly display name
	int Hash;						// Unique hash used to identify the zombie
	int Bounty[BOUNTY_TYPES];		// Money the player gets for certain things
	int XP[BOUNTY_TYPES];			// How much XP to get from certain things
	int Build;						// The zombie's body type, used for spawn
	int HeadHeight;					// How tall is this monster's head?
	int Height;						// Total height for the monster
	int TID;						// Monsters that spawn will use this TID
	bool Headless;					// This zombie cannot lose its head
	fixed HeadStrength;				// Multiplier for the monster's head damage
	
	bool CanFloor;					// Can we crawl from the floor?
	bool CanCeiling;				// Can we crawl from the ceiling?
	
	int MinimumWave;				// Which wave does this zed spawn on?
};

struct ZombieEntry 			ZombieList[ZLIST_SIZE];
int 						ZombieListSize;

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Add a brand new zombie to the list!
void RegisterZombie(str zClass, str zName, int zBounty, int zXP, int zWave = 1)
{
	ZombieList[ZombieListSize].Class = zClass;
	ZombieList[ZombieListSize].DisplayName = zName;
	
	ZombieList[ZombieListSize].Height = 56;
	ZombieList[ZombieListSize].HeadHeight = 13;
	
	ZombieList[ZombieListSize].HeadStrength = 1.0;
	
	ZombieList[ZombieListSize].Hash = FakeHash(zClass);
	
	ZombieList[ZombieListSize].MinimumWave = zWave;
	
	ZombieList[ZombieListSize].CanFloor = false;
	ZombieList[ZombieListSize].CanCeiling = false;
	
	ZombieList[ZombieListSize].Build = BUILD_HUMAN;
	
	// BASE amount of money to give
	ZombieList[ZombieListSize].Bounty[BOUNTY_MAIN] = zBounty;
	
	// These are added on
	ZombieList[ZombieListSize].Bounty[BOUNTY_HEADSHOT] = 30;
	ZombieList[ZombieListSize].Bounty[BOUNTY_CRIPPLE] = 20;
	ZombieList[ZombieListSize].Bounty[BOUNTY_GIB] = 50;
	
	// BASE experience to give
	ZombieList[ZombieListSize].XP[BOUNTY_MAIN] = zXP;
	
	// These are added on
	ZombieList[ZombieListSize].XP[BOUNTY_HEADSHOT] = 10;
	ZombieList[ZombieListSize].XP[BOUNTY_CRIPPLE] = 5;
	ZombieList[ZombieListSize].XP[BOUNTY_GIB] = 10;
	
	ZombieList[ZombieListSize].TID = TID_ZOMBIE;
	
	ZombieListSize ++;
}

// Set height values for the zombie
void RegisterZombieHeights(int bHeight, int hHeight)
{
	ZombieList[ZombieListSize-1].Height = bHeight;
	ZombieList[ZombieListSize-1].HeadHeight = hHeight;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Get the zombie's INDEX in the list by its hash
int ZombieByHash(int Hash)
{
	for (int l=0; l<ZombieListSize; l++)
	{
		if (ZombieList[l].Hash == Hash)
			return l;
	}
	
	return -1;
}

// Get the zombie's INDEX in the list by its class
int ZombieByType(str class)
{
	for (int l=0; l<ZombieListSize; l++)
	{
		if (!StrICmp(ZombieList[l].Class, class))
			return l;
	}
	
	return -1;
}

// With zombie as activator, get our index in the list
int GetZombieIndex()
{
	int ZI = CheckInventory("ZombieIndex");
	
	// If it's less than ZLIST_BASE then it wasn't initialized properly
	return (ZI < ZLIST_BASE) ? -1 : ZI - ZLIST_BASE;
}

// -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=