//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Z O M B I E   T O R R E N T   2 
// MAIN WEAPON LIST
//
// Contains info about EVERY weapon that can be selected
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Weapon upgrade struct!
// Contains information about a weapon's upgrades
struct WeaponUpgrade
{
	int Hash;						// Used for purchasing a specific upgrade
	str Name;						// Display name
	str Description;				// The description shown for this upgrade
	int Price;						// How much does it cost?
	str Item;						// The particular item to give, this is APPENDED to the wep name
	str Pic;						// Picture to show for this upgrade
};

// Main weapon struct, contains lots of info!
struct WeaponInfo 
{
	str Class;						// Internal class name of this particular weapon
	str DisplayName;				// Friendly display name, shown in trader menu etc.
	str Description;				// Description for this weapon, shown in trader
	str Picture;					// Picture shown in the trader menu
	int Crosshair;					// Default crosshair to use for this wep
	int Price;						// How much does this weapon cost?
	int Hash;						// Generated on C and S, both can refer to this number
	int DefaultFireMode;			// Default fire mode to use on HUD, unless otherwise specified
	
	bool Undroppable;				// Cannot drop this item
	
	str Perks[PERKBONUS_MAX];		// Array of perk bonuses that apply to this weapon
	int PerksSize;					// Length of array
	
	str PrimaryClass;				// Magazine ammunition class (primary)
	int PrimaryCap;					// Maximum amount of base PRIMARY ammo
	
	str SecondaryClass;				// Reserve ammunition class (secondary)
	int SecondaryCap;				// Maximum amount of base SECONDARY ammo
};

struct WeaponInfo WeaponList[MAX_WEPS];
int WeaponListSize;

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// Is this weapon perked for a specific perk index?
bool WeaponIsPerked(int WI, int PI)
{
	for (int i=0; i<WeaponList[WI].PerksSize; i++)
	{
		if (WeaponList[WI].Perks[i] == PI)
			return true;
	}
	
	return false;
}

// Get the weapon's INDEX in the list by its hash
int WeaponByHash(int Hash)
{
	for (int l=0; l<WeaponListSize; l++)
	{
		if (WeaponList[l].Hash == Hash)
			return l;
	}
	
	return -1;
}

// Get the weapon's INDEX in the list by its class
int WeaponByType(str class)
{
	for (int l=0; l<WeaponListSize; l++)
	{
		if (!StrICmp(WeaponList[l].class, class))
			return l;
	}
	
	return -1;
}

// Get the current weapon index from our player
// WARNING: CLIENT HAS NO KNOWLEDGE OF THIS (YET)

int LocalWeaponIndex()
{
	return curWeaponIndex[ PlayerNumber() ];
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// REGISTER A NEW WEAPON IN THE LIST
// Sets core properties and increases the list size
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeapon(str wClass, str wDisplay, int wPrice, str wPicture)
{
	WeaponList[WeaponListSize].Class = wClass;
	WeaponList[WeaponListSize].DisplayName = wDisplay;
	WeaponList[WeaponListSize].Price = wPrice;
	WeaponList[WeaponListSize].Picture = wPicture;
	WeaponList[WeaponListSize].Crosshair = XH_DEFAULT;
	WeaponList[WeaponListSize].Description = "No information about this weapon has been provided.";
	WeaponList[WeaponListSize].DefaultFireMode = FM_FULL;
	
	// Make unique identifier hash
	// (This may need to be combined with something else so it's unique)
	
	WeaponList[WeaponListSize].Hash = FakeHash(StrParam(s:wDisplay));
	
	WeaponListSize ++;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// SET WEAPON'S AMMUNITION AND AMOUNTS
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponAmmo(str wClassA, int wAmountA, str wClassB, int wAmountB)
{
	WeaponList[WeaponListSize-1].PrimaryClass = wClassA;
	WeaponList[WeaponListSize-1].PrimaryCap = wAmountA;
	WeaponList[WeaponListSize-1].SecondaryClass = wClassB;
	WeaponList[WeaponListSize-1].SecondaryCap = wAmountB;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// SET WEAPON'S DESCRIPTION
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponDescription(str wDesc)
{
	WeaponList[WeaponListSize-1].Description = wDesc;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// SET WEAPON AS UNDROPPABLE
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponUndroppable(bool value = true)
{
	WeaponList[WeaponListSize-1].Undroppable = value;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// SET DEFAULT WEAPON FIREMODE
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponDefaultFireMode(int FM = 0)
{
	WeaponList[WeaponListSize-1].DefaultFireMode = FM;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// SET DEFAULT WEAPON CROSSHAIR
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponCrosshair(int XH = 0)
{
	WeaponList[WeaponListSize-1].Crosshair = XH;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// ADD A PERK BONUS TO A PARTICULAR WEAPON
// Weapon index is optional, otherwise previous wep is used
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

void RegisterWeaponPerk(int pIndex, int wIndex = -1)
{
	int realIndex = (wIndex >= 0) ? wIndex : WeaponListSize-1;
	int cap = WeaponList[wIndex].PerksSize;
	
	WeaponList[realIndex].Perks[cap] = pIndex;
	WeaponList[realIndex].PerksSize ++;
}

//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

// PUKE THIS
script "MyGun" (void) NET CLIENTSIDE
{
	DebugLocalWeapon(0);
	NamedRequestScriptPuke("MyGunServer",0,0,0);
}

script "MyGunServer" NET
{
	DebugLocalWeapon(1);
}

void DebugLocalWeapon(int onServer)
{
	int idx = LocalWeaponIndex();
	if (idx < 0)
		return;
		
	str sideString;
		
	if (onServer)
		sideString = "[SERVER]";
	else
		sideString = "[CLIENT]";
		
	print(s:sideString, s:" YOUR GUN: ", s:WeaponList[idx].DisplayName, s:"\nREAL CLASS: ", s:GetWeapon(), s:"\nPRIMARY: ", d:CheckInventory("TestPrimary"));
}